@@ -20,7 +20,9 @@ from azureml.data.dataset_consumption_config import DatasetConsumptionConfig
 from azureml.dataprep.fuse.daemon import MountContext
 from azureml.exceptions._azureml_exception import UserErrorException
 
-from health_azure.utils import PathOrString, get_workspace, get_ml_client
+from health_azure.utils import PathOrString, get_ml_client
+
+logger = logging.getLogger(__name__)
 
 
 V1OrV2DataType = Union[FileDataset, Data]
@@ -35,6 +37,7 @@ def get_datastore(workspace: Workspace, datastore_name: str) -> Union[AzureBlobD
     :param datastore_name: The name of the datastore to retrieve.
     :return: An AzureML datastore.
     """
+
     def _retrieve_v1_datastore(datastores: Dict[str, Datastore], datastore_name: str) -> Datastore:
         # First check if there is only one datastore, which is then obviously unique.
         # Only then try to use the default datastore, because there may not be a default set.
@@ -48,8 +51,10 @@ def get_datastore(workspace: Workspace, datastore_name: str) -> Union[AzureBlobD
 
         if datastore_name in datastores:
             return datastores[datastore_name]
-        raise ValueError(f"Datastore \"{datastore_name}\" was not found in the \"{workspace.name}\" workspace. "
-                         f"Existing datastores: {existing_stores}")
+        raise ValueError(
+            f"Datastore \"{datastore_name}\" was not found in the \"{workspace.name}\" workspace. "
+            f"Existing datastores: {existing_stores}"
+        )
 
     def _retrieve_v2_datastore(datastores: DatastoreOperations, datastore_name: str) -> V2Datastore:
         existing_stores = list(datastores.list())
@@ -88,8 +93,7 @@ def _retrieve_v1_dataset(dataset_name: str, workspace: Workspace) -> Optional[Fi
     return azureml_dataset
 
 
-def _create_v1_dataset(datastore_name: str, dataset_name: str, workspace: Workspace
-                       ) -> FileDataset:
+def _create_v1_dataset(datastore_name: str, dataset_name: str, workspace: Workspace) -> FileDataset:
     """
     Create a v1 Dataset in the specified Datastore
 
@@ -126,6 +130,14 @@ def _get_or_create_v1_dataset(datastore_name: str, dataset_name: str, workspace:
     try:
         azureml_dataset = _retrieve_v1_dataset(dataset_name, workspace)
     except UserErrorException:
+        logger.warning(f"Dataset '{dataset_name}' was not found, or is not an AzureML SDK v1 dataset.")
+        logger.info(f"Trying to create a new dataset '{dataset_name}' from files in folder '{dataset_name}'")
+        if datastore_name == "":
+            raise ValueError(
+                "When creating a new dataset, a datastore name must be provided. Please specify a datastore name using "
+                "the --datastore flag"
+            )
+        logger.info(f"Trying to create a new dataset '{dataset_name}' in datastore '{datastore_name}'")
         azureml_dataset = _create_v1_dataset(datastore_name, dataset_name, workspace)
     return azureml_dataset
 
@@ -182,10 +194,7 @@ def _retrieve_v2_data_asset(
 
 
 def _create_v2_data_asset(
-    ml_client: MLClient,
-    datastore_name: str,
-    data_asset_name: str,
-    version: Optional[str] = None
+    ml_client: MLClient, datastore_name: str, data_asset_name: str, version: Optional[str] = None
 ) -> Data:
     """
     Create or update a v2 Data asset in the specified Datastore
@@ -194,15 +203,17 @@ def _create_v2_data_asset(
     :param datastore_name: The name of the datastore in which to create or update the Data asset.
     :param data_asset_name: The name of the data asset to be created.
     :param version: The version of the data asset to be created.
+    :raises ValueError: If no data asset name is provided to define the new asset.
     :raises ValueError: If no datastore name is provided to define where to create the data.
     :return: The created or updated Data asset.
     """
-    if not data_asset_name:
+    if data_asset_name == "":
         raise ValueError("Cannot create data asset with empty name.")
 
-    if not datastore_name:
-        default_datastore = ml_client.datastores.get_default()
-        datastore_name = default_datastore.name
+    if datastore_name == "":
+        raise ValueError(
+            "Cannot create data asset with empty datastore name. Please specify a datastore name using the --datastore flag."
+        )
 
     logging.info(
         f"Creating a new Data asset from data in folder '{data_asset_name}' in the datastore '{datastore_name}'"
@@ -220,10 +231,7 @@ def _create_v2_data_asset(
 
 
 def _get_or_create_v2_data_asset(
-    ml_client: MLClient,
-    datastore_name: str,
-    data_asset_name: str,
-    version: Optional[str] = None
+    ml_client: MLClient, datastore_name: str, data_asset_name: str, version: Optional[str] = None
 ) -> Data:
     """
     Attempt to retrieve a v2 data asset object and return that, otherwise attempt to create and register
@@ -246,12 +254,13 @@ def _get_or_create_v2_data_asset(
     return azureml_data_asset
 
 
-def get_or_create_dataset(datastore_name: str,
-                          dataset_name: str,
-                          workspace: Workspace,
-                          strictly_aml_v1: bool,
-                          ml_client: Optional[MLClient] = None,
-                          ) -> V1OrV2DataType:
+def get_or_create_dataset(
+    datastore_name: str,
+    dataset_name: str,
+    workspace: Workspace,
+    strictly_aml_v1: bool,
+    ml_client: Optional[MLClient] = None,
+) -> V1OrV2DataType:
     """
     Looks in the AzureML datastore for a dataset of the given name. If there is no such dataset, a dataset is
     created and registered, assuming that the files are in a folder that has the same name as the dataset.
@@ -300,18 +309,23 @@ class DatasetConfig:
     Contains information to use AzureML datasets as inputs or outputs.
     """
 
-    def __init__(self,
-                 name: str,
-                 datastore: str = "",
-                 version: Optional[int] = None,
-                 use_mounting: Optional[bool] = None,
-                 target_folder: Optional[PathOrString] = None,
-                 local_folder: Optional[PathOrString] = None):
+    def __init__(
+        self,
+        name: str,
+        datastore: str = "",
+        overwrite_existing: bool = True,
+        version: Optional[int] = None,
+        use_mounting: Optional[bool] = None,
+        target_folder: Optional[PathOrString] = None,
+        local_folder: Optional[PathOrString] = None,
+    ):
         """
         :param name: The name of the dataset, as it was registered in the AzureML workspace. For output datasets,
             this will be the name given to the newly created dataset.
         :param datastore: The name of the AzureML datastore that holds the dataset. This can be empty if the AzureML
             workspace has only a single datastore, or if the default datastore should be used.
+        :param overwrite_existing: Only applies to uploading datasets. If True, the dataset will be overwritten if it
+            already exists. If False, the dataset creation will fail if the dataset already exists.
         :param version: The version of the dataset that should be used. This is only used for input datasets.
             If the version is not specified, the latest version will be used.
         :param use_mounting: If True, the dataset will be "mounted", that is, individual files will be read
@@ -332,6 +346,7 @@ class DatasetConfig:
             raise ValueError("The name of the dataset must be a non-empty string.")
         self.name = name
         self.datastore = datastore
+        self.overwrite_existing = overwrite_existing
         self.version = version
         self.use_mounting = use_mounting
         # If target_folder is "" then convert to None
@@ -340,11 +355,10 @@ class DatasetConfig:
             raise ValueError("Can't mount or download a dataset to the current working directory.")
         self.local_folder = Path(local_folder) if local_folder else None
 
-    def to_input_dataset_local(self,
-                               strictly_aml_v1: bool,
-                               workspace: Workspace = None,
-                               ml_client: Optional[MLClient] = None,
-                               ) -> Tuple[Optional[Path], Optional[MountContext]]:
+    def to_input_dataset_local(
+        self,
+        workspace: Workspace,
+    ) -> Tuple[Path, Optional[MountContext]]:
         """
         Return a local path to the dataset when outside of an AzureML run.
         If local_folder is supplied, then this is assumed to be a local dataset, and this is returned.
@@ -353,9 +367,6 @@ class DatasetConfig:
         therefore a tuple of Nones will be returned.
 
         :param workspace: The AzureML workspace to read from.
-        :param strictly_aml_v1: If True, use Azure ML SDK v1 to attempt to find or create and reigster the dataset.
-            Otherwise, attempt to use Azure ML SDK v2.
-        :param ml_client: An Azure MLClient object for interacting with Azure resources.
         :return: Tuple of (path to dataset, optional mountcontext)
         """
         status = f"Dataset '{self.name}' will be "
@@ -366,14 +377,14 @@ class DatasetConfig:
             return self.local_folder, None
 
         if workspace is None:
-            raise ValueError(f"Unable to make dataset '{self.name} available for a local run because no AzureML "
-                             "workspace has been provided. Provide a workspace, or set a folder for local execution.")
-        azureml_dataset = get_or_create_dataset(
+            raise ValueError(
+                f"Unable to make dataset '{self.name} available for a local run because no AzureML "
+                "workspace has been provided. Provide a workspace, or set a folder for local execution."
+            )
+        azureml_dataset = _get_or_create_v1_dataset(
             datastore_name=self.datastore,
             dataset_name=self.name,
             workspace=workspace,
-            strictly_aml_v1=strictly_aml_v1,
-            ml_client=ml_client,
         )
         if isinstance(azureml_dataset, FileDataset):
             target_path = self.target_folder or Path(tempfile.mkdtemp())
@@ -391,14 +402,15 @@ class DatasetConfig:
             print(status)
             return result
         else:
-            return None, None
+            raise ValueError(f"Don't know how to handle dataset '{self.name}' of type {type(azureml_dataset)}")
 
-    def to_input_dataset(self,
-                         dataset_index: int,
-                         workspace: Workspace,
-                         strictly_aml_v1: bool,
-                         ml_client: Optional[MLClient] = None,
-                         ) -> Optional[DatasetConsumptionConfig]:
+    def to_input_dataset(
+        self,
+        dataset_index: int,
+        workspace: Workspace,
+        strictly_aml_v1: bool,
+        ml_client: Optional[MLClient] = None,
+    ) -> Optional[DatasetConsumptionConfig]:
         """
         Creates a configuration for using an AzureML dataset inside of an AzureML run. This will make the AzureML
         dataset with given name available as a named input, using INPUT_0 as the key for dataset index 0.
@@ -437,21 +449,20 @@ class DatasetConfig:
         else:
             return None
 
-    def to_output_dataset(self,
-                          workspace: Workspace,
-                          dataset_index: int) -> OutputFileDatasetConfig:
+    def to_output_dataset(self, workspace: Workspace, dataset_index: int) -> OutputFileDatasetConfig:
         """
         Creates a configuration to write a script output to an AzureML dataset. The name and datastore of this new
         dataset will be taken from the present object.
 
         :param workspace: The AzureML workspace to read from.
         :param dataset_index: Suffix for using datasets as named inputs, the dataset will be marked OUTPUT_{index}
-        :return:
+        :return: An AzureML OutputFileDatasetConfig object, representing the output dataset.
         """
         status = f"Output dataset {self.name} (index {dataset_index}) will be "
         datastore = get_datastore(workspace, self.datastore)
-        dataset = OutputFileDatasetConfig(name=_output_dataset_key(index=dataset_index),
-                                          destination=(datastore, self.name + "/"))
+        dataset = OutputFileDatasetConfig(
+            name=_output_dataset_key(index=dataset_index), destination=(datastore, self.name + "/")
+        )
         # TODO: Can we get tags into here too?
         dataset = dataset.register_on_complete(name=self.name)
         if self.target_folder:
@@ -462,7 +473,7 @@ class DatasetConfig:
             result = dataset.as_mount()
         else:
             status += "uploaded when the job completes."
-            result = dataset.as_upload()
+            result = dataset.as_upload(overwrite=self.overwrite_existing)
         logging.info(status)
         return result
 
@@ -470,8 +481,7 @@ class DatasetConfig:
 StrOrDatasetConfig = Union[str, DatasetConfig]
 
 
-def _replace_string_datasets(datasets: List[StrOrDatasetConfig],
-                             default_datastore_name: str) -> List[DatasetConfig]:
+def _replace_string_datasets(datasets: List[StrOrDatasetConfig], default_datastore_name: str) -> List[DatasetConfig]:
     """
     Processes a list of input or output datasets. All entries in the list that are strings are turned into
     DatasetConfig objects, using the string as the dataset name, and pointing to the default datastore.
@@ -480,15 +490,16 @@ def _replace_string_datasets(datasets: List[StrOrDatasetConfig],
     :param default_datastore_name: The datastore to use for all datasets that are only specified via their name.
     :return: A list of DatasetConfig objects, in the same order as the input list.
     """
-    return [DatasetConfig(name=d, datastore=default_datastore_name) if isinstance(d, str) else d
-            for d in datasets]
+    return [DatasetConfig(name=d, datastore=default_datastore_name) if isinstance(d, str) else d for d in datasets]
 
 
-def create_dataset_configs(all_azure_dataset_ids: List[str],
-                           all_dataset_mountpoints: Sequence[PathOrString],
-                           all_local_datasets: List[Optional[Path]],
-                           datastore: Optional[str] = None,
-                           use_mounting: bool = False) -> List[DatasetConfig]:
+def create_dataset_configs(
+    all_azure_dataset_ids: List[str],
+    all_dataset_mountpoints: Sequence[PathOrString],
+    all_local_datasets: List[Optional[Path]],
+    datastore: Optional[str] = None,
+    use_mounting: bool = False,
+) -> List[DatasetConfig]:
     """
     Sets up all the dataset consumption objects for the datasets provided. The returned list will have the same length
     as there are non-empty azure dataset IDs.
@@ -518,58 +529,35 @@ def create_dataset_configs(all_azure_dataset_ids: List[str],
         count = num_azure
     elif num_azure == 0 and num_mount == 0:
         # No datasets in Azure at all: This is possible for runs that for example download their own data from the web.
-        # There can be any number of local datasets, but we are not checking that. In MLRunner.setup, there is a check
+        # There can be any number of local datasets, but we are not checking that. In TrainingRunner.setup, there is a check
         # that leaves local datasets intact if there are no Azure datasets.
         return []
     else:
-        raise ValueError("Invalid dataset setup. You need to specify N entries in azure_datasets and a matching "
-                         "number of local_datasets and dataset_mountpoints")
+        raise ValueError(
+            "Invalid dataset setup. You need to specify N entries in azure_datasets and a matching "
+            "number of local_datasets and dataset_mountpoints"
+        )
     for i in range(count):
         azure_dataset = all_azure_dataset_ids[i] if i < num_azure else ""
         if not azure_dataset:
             continue
         mount_point = all_dataset_mountpoints[i] if i < num_mount else ""
         local_dataset = all_local_datasets[i] if i < num_local else None
-        config = DatasetConfig(name=azure_dataset,
-                               target_folder=mount_point,
-                               local_folder=local_dataset,
-                               use_mounting=use_mounting,
-                               datastore=datastore or "")
+        config = DatasetConfig(
+            name=azure_dataset,
+            target_folder=mount_point,
+            local_folder=local_dataset,
+            use_mounting=use_mounting,
+            datastore=datastore or "",
+        )
         datasets.append(config)
     return datasets
 
 
-def find_workspace_for_local_datasets(aml_workspace: Optional[Workspace],
-                                      workspace_config_path: Optional[Path],
-                                      dataset_configs: List[DatasetConfig]) -> Optional[Workspace]:
-    """
-    If any of the dataset_configs require an AzureML workspace then try to get one, otherwise return None.
-
-    :param aml_workspace: There are two optional parameters used to glean an existing AzureML Workspace. The simplest is
-        to pass it in as a parameter.
-    :param workspace_config_path: The 2nd option is to specify the path to the config.json file downloaded from the
-        Azure portal from which we can retrieve the existing Workspace.
-    :param dataset_configs: List of DatasetConfig describing the input datasets.
-    :return: Workspace if required, None otherwise.
-    """
-    workspace: Workspace = None
-    # Check whether an attempt will be made to mount or download a dataset when running locally.
-    # If so, try to get the AzureML workspace.
-    if any(dc.local_folder is None for dc in dataset_configs):
-        try:
-            workspace = get_workspace(aml_workspace, workspace_config_path)
-            logging.info(f"Found workspace for datasets: {workspace.name}")
-        except Exception as ex:
-            logging.info(f"Could not find workspace for datasets. Exception: {ex}")
-    return workspace
-
-
-def setup_local_datasets(dataset_configs: List[DatasetConfig],
-                         strictly_aml_v1: bool,
-                         aml_workspace: Optional[Workspace] = None,
-                         ml_client: Optional[MLClient] = None,
-                         workspace_config_path: Optional[Path] = None,
-                         ) -> Tuple[List[Optional[Path]], List[MountContext]]:
+def setup_local_datasets(
+    dataset_configs: List[DatasetConfig],
+    workspace: Optional[Workspace],
+) -> Tuple[List[Path], List[MountContext]]:
     """
     When running outside of AzureML, setup datasets to be used locally.
 
@@ -577,21 +565,20 @@ def setup_local_datasets(dataset_configs: List[DatasetConfig],
     used. Otherwise the dataset is mounted or downloaded to either the target folder or a temporary folder and that is
     used.
 
-    :param aml_workspace: There are two optional parameters used to glean an existing AzureML Workspace. The simplest is
-        to pass it in as a parameter.
-    :param workspace_config_path: The 2nd option is to specify the path to the config.json file downloaded from the
-        Azure portal from which we can retrieve the existing Workspace.
+    If a dataset does not exist, an AzureML SDK v1 dataset will be created, assuming that the dataset is given
+    in a folder of the same name (for example, if a dataset is given as "mydataset", then it is created from the files
+    in folder "mydataset" in the datastore).
+
+    :param workspace: The AzureML workspace to work with. Can be None if the list of datasets is empty, or if
+        the datasets are available local.
     :param dataset_configs: List of DatasetConfig describing the input data assets.
-    :param strictly_aml_v1: If True, use Azure ML SDK v1. Otherwise, attempt to use Azure ML SDK v2.
-    :param ml_client: An MLClient object for interacting with AML v2 datastores.
-    :return: Pair of: list of optional paths to the input datasets, list of mountcontexts, one for each mounted dataset.
+    :return: Pair of: list of paths to the input datasets, list of mountcontexts, one for each mounted dataset.
     """
-    workspace = find_workspace_for_local_datasets(aml_workspace, workspace_config_path, dataset_configs)
-    mounted_input_datasets: List[Optional[Path]] = []
+    mounted_input_datasets: List[Path] = []
     mount_contexts: List[MountContext] = []
 
     for data_config in dataset_configs:
-        target_path, mount_context = data_config.to_input_dataset_local(strictly_aml_v1, workspace, ml_client)
+        target_path, mount_context = data_config.to_input_dataset_local(workspace)
 
         mounted_input_datasets.append(target_path)
 
